<?php
	/**
	 * @author			Matthias Reuter
	 * @copyright		2017 Matthias Reuter
	 * @link			https://straightvisions.com/
	 * @since			1.0
	 * @license			This is no free software. See license.txt or https://straightvisions.com/
	 */

	namespace sv_gravity_forms_enhancer;

	abstract class sv_plugin{
		protected static $name						= false;
		protected static $module_name				= false;
		protected static $basename					= false;
		protected static $path						= false;
		protected static $url						= false;
		protected static $version					= false;
		protected static $wpdb						= false;

		/**
		 * @desc			Load's requested libraries dynamicly
		 * @param	string	$name library-name
		 * @return			class object of the requested library
		 * @author			Matthias Reuter
		 * @since			1.0
		 * @ignore
		 */
		public function __get(string $name){
			if($this->get_path_lib_modules($name.'.php')){ // look for class file in modules directory
				require_once($this->get_path_lib_modules($name.'.php'));
				$classname							= self::$name.'\\'.$name;
				$this->$name						= new $classname();
				return $this->$name;
			}else{
				throw new \Exception('Class '.$name.' could not be loaded (tried to load class-file '.$this->get_path_lib_modules($name.'.php').')');
			}
		}
		/**
		 * @desc			initialize plugin
		 * @author			Matthias Reuter
		 * @since			1.0
		 * @ignore
		 */
		public function __construct($path=false){
			$this->setup($path);
			$this->init();
			$this->start();
		}
		protected function setup($path){
			self::$name								= __NAMESPACE__;
			self::$module_name						= (new \ReflectionClass($this))->getShortName();
			if($path){
				self::$path								= $path;
			}
			self::$url								= trailingslashit(plugins_url('', self::$path.'self::$name'));
			self::$version							= 1000;
			
			global $wpdb;
			self::$wpdb								= $wpdb;
			
			load_plugin_textdomain(self::$name, false, dirname(plugin_basename(__FILE__)).'/lib/lang/');
		}
		protected function init(){
			
		}
		protected function start(){
			
		}
		public function get_name(){
			return self::$name;
		}
		public function get_module_name(){
			return self::$module_name;
		}
		public function get_version(){
			return self::$version;
		}
		
		public function get_path($suffix=''){
			if(file_exists(self::$path.$suffix)){
				return self::$path.$suffix;
			}else{
				return false;
			}
		}
		public function get_url($suffix=''){
			if(file_exists(self::$path.$suffix)){
				return self::$url.$suffix;
			}else{
				return false;
			}
		}
		public function get_path_lib($suffix=''){
			if(file_exists($this->get_path('lib/').$suffix)){
				return $this->get_path('lib/').$suffix;
			}else{
				return false;
			}
		}
		public function get_url_lib($suffix=''){
			if(file_exists($this->get_path('lib/').$suffix)){
				return $this->get_url('lib/').$suffix;
			}else{
				return false;
			}
		}
		public function get_path_lib_modules($suffix=''){
			if(file_exists($this->get_path_lib('modules/').$suffix)){
				return $this->get_path_lib('modules/').$suffix;
			}else{
				return false;
			}
		}
		/*
			default hierarchy:
			/lib/frontend/(img|css|js|tpl)/
			/lib/backend/(img|css|js|tpl)/
		*/
		public function get_path_lib_section($section='frontend',$dir='',$suffix=''){
			if(file_exists($this->get_path_lib(trailingslashit($section).trailingslashit($dir)))){
				return $this->get_path_lib(trailingslashit($section).trailingslashit($dir)).$suffix;
			}else{
				return false;
			}
		}
		public function get_url_lib_section($section='frontend',$dir='',$suffix=''){
			if(file_exists($this->get_path_lib(trailingslashit($section).trailingslashit($dir)))){
				return $this->get_url_lib(trailingslashit($section).trailingslashit($dir)).$suffix;
			}else{
				return false;
			}
		}
	}
?>